@@ -11,11 +11,7 @@ use sha2::{Digest, Sha256};
 use std::sync::Arc;
 
 /// Parse a user agent string.
-///
-/// # Stability
-///
-/// This is part of an experimental API that is subject to change or removal even in minor versions
-/// of this crate.
+#[doc = include_str!("../docs/snippets/experimental.md")]
 pub fn uap_parse(
     user_agent: &str,
 ) -> Result<(String, Option<String>, Option<String>, Option<String>), Error> {
@@ -88,94 +84,50 @@ pub fn uap_parse(
 }
 
 /// An extension trait for [`Request`]s that adds methods for controlling cache keys.
-///
-/// # Stability
-///
-/// This is part of an experimental API that is subject to change or removal even in minor versions
-/// of this crate.
+#[doc = include_str!("../docs/snippets/experimental.md")]
 pub trait RequestCacheKey {
     /// See [`Request::set_cache_key()`].
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn set_cache_key(&mut self, key: [u8; 32]);
     /// See [`Request::with_cache_key()`].
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn with_cache_key(self, key: [u8; 32]) -> Self;
     /// See [`Request::set_cache_key_fn()`].
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn set_cache_key_fn(&mut self, f: impl Fn(&Request) -> [u8; 32] + Send + Sync + 'static);
     /// See [`Request::with_cache_key_fn()`].
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn with_cache_key_fn(self, f: impl Fn(&Request) -> [u8; 32] + Send + Sync + 'static) -> Self;
     /// See [`Request::set_cache_key_str()`].
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn set_cache_key_str(&mut self, key_str: impl AsRef<[u8]>);
     /// See [`Request::with_cache_key_str()`].
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn with_cache_key_str(self, key_str: impl AsRef<[u8]>) -> Self;
 }
 
 impl RequestCacheKey for Request {
     /// Set the cache key to be used when attempting to satisfy this request from a cached response.
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn set_cache_key(&mut self, key: [u8; 32]) {
         self.cache_key = Some(CacheKeyGen::Set(key));
     }
 
     /// Builder-style equivalent of [`set_cache_key()`](Self::set_cache_key()).
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn with_cache_key(mut self, key: [u8; 32]) -> Self {
         self.set_cache_key(key);
         self
     }
 
     /// Set the function that will be used to compute the cache key for this request.
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn set_cache_key_fn(&mut self, f: impl Fn(&Request) -> [u8; 32] + Send + Sync + 'static) {
         self.cache_key = Some(CacheKeyGen::Lazy(Arc::new(f)));
     }
 
     /// Builder-style equivalent of [`set_cache_key_fn()`](Self::set_cache_key_fn()).
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn with_cache_key_fn(
         mut self,
         f: impl Fn(&Request) -> [u8; 32] + Send + Sync + 'static,
@@ -189,11 +141,7 @@ impl RequestCacheKey for Request {
     ///
     /// The string representation of the key is hashed to the same `[u8; 32]` representation used by
     /// [`set_cache_key()`][`Self::set_cache_key()`].
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn set_cache_key_str(&mut self, key_str: impl AsRef<[u8]>) {
         let mut sha = Sha256::new();
         sha.update(key_str);
@@ -202,11 +150,7 @@ impl RequestCacheKey for Request {
     }
 
     /// Builder-style equivalent of [`set_cache_key_str()`](Self::set_cache_key_str()).
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn with_cache_key_str(mut self, key_str: impl AsRef<[u8]>) -> Self {
         self.set_cache_key_str(key_str);
         self
@@ -215,28 +159,16 @@ impl RequestCacheKey for Request {
 
 /// An extension trait for [`RequestHandle`](RequestHandle)s that adds methods for controlling cache
 /// keys.
-///
-/// # Stability
-///
-/// This is part of an experimental API that is subject to change or removal even in minor versions
-/// of this crate.
+#[doc = include_str!("../docs/snippets/experimental.md")]
 pub trait RequestHandleCacheKey {
     /// See [`RequestHandle::set_cache_key()`](RequestHandle::set_cache_key).
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn set_cache_key(&mut self, key: &[u8; 32]);
 }
 
 impl RequestHandleCacheKey for RequestHandle {
     /// Set the cache key to be used when attempting to satisfy this request from a cached response.
-    ///
-    /// # Stability
-    ///
-    /// This is part of an experimental API that is subject to change or removal even in minor
-    /// versions of this crate.
+    #[doc = include_str!("../docs/snippets/experimental.md")]
     fn set_cache_key(&mut self, key: &[u8; 32]) {
         const DIGITS: &[u8; 16] = b"0123456789ABCDEF";
         let mut hex = [0; 64];
