@@ -1,6 +1,6 @@
 //! Geographic data for IP addresses.
 
-pub use chrono::FixedOffset;
+pub use time::UtcOffset;
 
 use crate::abi::{self, FastlyStatus};
 use crate::error::BufferSizeError;
@@ -25,6 +25,15 @@ use std::net::IpAddr;
 /// }
 /// ```
 pub fn geo_lookup(ip: IpAddr) -> Option<Geo> {
+    geo_lookup_raw(ip).map(Geo::from_raw)
+}
+
+/// Look up the raw geographic data associated with a particular IP address.
+///
+/// Returns `None` if no geographic data is available, such as when the IP address is reserved for
+/// private use. The returned `RawGeo` may contain fields with Fastly-documented error values,
+/// which the higher-level `Geo` will translate to `Option` or `Result`.
+fn geo_lookup_raw(ip: IpAddr) -> Option<RawGeo> {
     use std::net::IpAddr::{V4, V6};
     let (addr_bytes, addr_len) = match ip {
         V4(ip) => (ip.octets().to_vec(), 4),
@@ -39,11 +48,7 @@ pub fn geo_lookup(ip: IpAddr) -> Option<Geo> {
     };
 
     // Try to parse any non-null response, returning `None` otherwise.
-    result.and_then(|geo_bytes| {
-        serde_json::from_slice::<'_, RawGeo>(&geo_bytes)
-            .ok()
-            .and_then(Geo::try_from_raw)
-    })
+    result.and_then(|geo_bytes| serde_json::from_slice::<'_, RawGeo>(&geo_bytes).ok())
 }
 
 pub(crate) fn geo_lookup_impl(
@@ -78,14 +83,21 @@ pub(crate) fn geo_lookup_impl(
     }
 }
 
+/// The raw geographic data associated with a particular IP address.
+///
+/// Users of the `fastly` crate likely want the [`geo::Geo`] struct instead, from
+/// [`geo::geo_lookup`], which will handle some error modes possible in the raw result.
 #[derive(Clone, Debug, Deserialize, Serialize)]
 struct RawGeo {
     as_name: String,
     as_number: u32,
     area_code: u16,
     city: String,
+    #[serde(deserialize_with = "deserialize_conn_speed")]
     conn_speed: ConnSpeed,
+    #[serde(deserialize_with = "deserialize_conn_type")]
     conn_type: ConnType,
+    #[serde(deserialize_with = "deserialize_continent")]
     continent: Continent,
     country_code: String,
     country_code3: String,
@@ -94,7 +106,9 @@ struct RawGeo {
     longitude: f64,
     metro_code: i64,
     postal_code: String,
+    #[serde(deserialize_with = "deserialize_proxy_description")]
     proxy_description: ProxyDescription,
+    #[serde(deserialize_with = "deserialize_proxy_type")]
     proxy_type: ProxyType,
     region: Option<String>,
     utc_offset: i32,
@@ -127,15 +141,19 @@ pub struct Geo {
     proxy_description: ProxyDescription,
     proxy_type: ProxyType,
     region: Option<String>,
-    utc_offset: FixedOffset,
+    utc_offset: Option<UtcOffset>,
 }
 
 impl Geo {
-    fn try_from_raw(raw: RawGeo) -> Option<Self> {
-        let hours = raw.utc_offset / 100;
-        let minutes = raw.utc_offset % 100;
-        let seconds = (hours * 60 * 60) + (minutes * 60);
-        FixedOffset::east_opt(seconds).map(|utc_offset| Geo {
+    fn from_raw(raw: RawGeo) -> Self {
+        let utc_offset = if raw.utc_offset != 9999 {
+            let hours = (raw.utc_offset as u16 / 100) as i8;
+            let minutes = (raw.utc_offset as u16 % 100) as i8;
+            UtcOffset::from_hms(hours, minutes, 0).ok()
+        } else {
+            None
+        };
+        Geo {
             as_name: raw.as_name,
             as_number: raw.as_number,
             area_code: raw.area_code,
@@ -154,7 +172,7 @@ impl Geo {
             proxy_type: raw.proxy_type,
             region: raw.region,
             utc_offset,
-        })
+        }
     }
 
     /// The name of the organization associated with `as_number`.
@@ -183,17 +201,17 @@ impl Geo {
 
     /// Connection speed.
     pub fn conn_speed(&self) -> ConnSpeed {
-        self.conn_speed
+        self.conn_speed.clone()
     }
 
     /// Connection type.
     pub fn conn_type(&self) -> ConnType {
-        self.conn_type
+        self.conn_type.clone()
     }
 
     /// Continent.
     pub fn continent(&self) -> Continent {
-        self.continent
+        self.continent.clone()
     }
 
     /// A two-character [ISO 3166-1][iso] country code for the country associated with an IP address.
@@ -268,12 +286,12 @@ impl Geo {
 
     /// Client proxy description.
     pub fn proxy_description(&self) -> ProxyDescription {
-        self.proxy_description
+        self.proxy_description.clone()
     }
 
     /// Client proxy type.
     pub fn proxy_type(&self) -> ProxyType {
-        self.proxy_type
+        self.proxy_type.clone()
     }
 
     /// [ISO 3166-2][iso] country subdivision code.
@@ -318,10 +336,13 @@ impl Geo {
 
     /// Time zone offset from coordinated universal time (UTC) for `city`.
     ///
-    /// This is represented using the [`FixedOffset`] type from the [`chrono`] library. See
-    /// `chrono`'s documentation for more details on how to format this type or use it in time
+    /// This is represented using the [`UtcOffset`] type from the [`time`] library. See
+    /// `time`'s documentation for more details on how to format this type or use it in time
     /// calculations.
-    pub fn utc_offset(&self) -> FixedOffset {
+    ///
+    /// Returns `None` if the geolocation database does not have a time zone offset for this IP
+    /// address.
+    pub fn utc_offset(&self) -> Option<UtcOffset> {
         self.utc_offset
     }
 }
@@ -335,8 +356,9 @@ impl Geo {
 /// [oc]: https://en.wikipedia.org/wiki/Optical_Carrier_transmission_rates
 /// [t]: https://en.wikipedia.org/wiki/T-carrier
 #[allow(missing_docs)]
+#[derive(Clone, Debug, Deserialize, Serialize, PartialEq, Eq)]
 #[serde(rename_all = "kebab-case")]
-#[derive(Clone, Copy, Debug, Deserialize, Serialize, PartialEq, Eq)]
+#[non_exhaustive]
 pub enum ConnSpeed {
     Broadband,
     Cable,
@@ -344,33 +366,45 @@ pub enum ConnSpeed {
     Mobile,
     Oc12,
     Oc3,
+    Satellite,
     T1,
     T3,
-    Satellite,
+    #[serde(rename = "ultrabb")]
+    UltraBroadband,
     Wireless,
     Xdsl,
+    /// A network connection speed that is known, but not in the above list of variants.
+    ///
+    /// This typically indicates that the geolocation database contains a connection speed
+    /// that did not exist when this crate was published.
+    Other(String),
 }
 
 /// Connection type.
 ///
 /// Defaults to `Unknown` when the connection type is not known.
 #[allow(missing_docs)]
+#[derive(Clone, Debug, Deserialize, Serialize, PartialEq, Eq)]
 #[serde(rename_all = "kebab-case")]
-#[derive(Clone, Copy, Debug, Deserialize, Serialize, PartialEq, Eq)]
+#[non_exhaustive]
 pub enum ConnType {
     Wired,
     Wifi,
     Mobile,
     Dialup,
     Satellite,
-    #[serde(other)]
     #[serde(rename = "?")]
     Unknown,
+    /// A type of network connection that is known, but not in the above list of variants.
+    ///
+    /// This typically indicates that the geolocation database contains a connection type
+    /// that did not exist when this crate was published.
+    Other(String),
 }
 
 /// Continent.
 #[allow(missing_docs)]
-#[derive(Clone, Copy, Debug, Deserialize, Serialize, PartialEq, Eq)]
+#[derive(Clone, Debug, Deserialize, Serialize, PartialEq, Eq)]
 pub enum Continent {
     #[serde(rename = "AF")]
     Africa,
@@ -386,6 +420,12 @@ pub enum Continent {
     Oceania,
     #[serde(rename = "SA")]
     SouthAmerica,
+    /// A continent that is known, but not one of the above variants.
+    ///
+    /// The Earth is not prone to spontaneously developing new continents, however *names* of
+    /// continents might change. If the short name for a continent changes, this is how an unknown
+    /// name would be reported.
+    Other(String),
 }
 
 impl Continent {
@@ -400,6 +440,9 @@ impl Continent {
     /// | South America | `SA` |
     /// | Oceania       | `OC` |
     /// | Antarctica    | `AN` |
+    ///
+    /// In the case of an unrecognized continent code in the geolocation database, `as_code` may
+    /// return `??`.
     pub fn as_code(&self) -> &'static str {
         match self {
             Self::Africa => "AF",
@@ -409,6 +452,7 @@ impl Continent {
             Self::NorthAmerica => "NA",
             Self::Oceania => "OC",
             Self::SouthAmerica => "SA",
+            Self::Other(_) => "??",
         }
     }
 }
@@ -416,8 +460,9 @@ impl Continent {
 /// Client proxy description.
 ///
 /// Defaults to `Unknown` when an IP address is not known to be a proxy or VPN.
+#[derive(Clone, Debug, Deserialize, Serialize, PartialEq, Eq)]
 #[serde(rename_all = "kebab-case")]
-#[derive(Clone, Copy, Debug, Deserialize, Serialize, PartialEq, Eq)]
+#[non_exhaustive]
 pub enum ProxyDescription {
     /// Enables ubiquitous network access to a shared pool of configurable computing resources.
     Cloud,
@@ -441,9 +486,13 @@ pub enum ProxyDescription {
     /// Examples of such browsers are Opera mobile browsers and UCBrowser.
     WebBrowser,
     /// An IP address that is not known to be a proxy or VPN.
-    #[serde(other)]
     #[serde(rename = "?")]
     Unknown,
+    /// Description of a proxy or VPN that is known, but not in the above list of variants.
+    ///
+    /// This typically indicates that the geolocation database contains a proxy description that
+    /// did not exist when this crate was published.
+    Other(String),
 }
 
 /// Client proxy type.
@@ -452,8 +501,9 @@ pub enum ProxyDescription {
 // TODO ACF 2020-04-22: the docs on https://docs.fastly.com/vcl/variables/client-geo-proxy-type/
 // look like they need a refresher, so I did not transcribe them for the individual variants.
 #[allow(missing_docs)]
+#[derive(Clone, Debug, Deserialize, Serialize, PartialEq, Eq)]
 #[serde(rename_all = "kebab-case")]
-#[derive(Clone, Copy, Debug, Deserialize, Serialize, PartialEq, Eq)]
+#[non_exhaustive]
 pub enum ProxyType {
     Anonymous,
     Aol,
@@ -463,7 +513,114 @@ pub enum ProxyType {
     Hosting,
     Public,
     Transparent,
-    #[serde(other)]
     #[serde(rename = "?")]
     Unknown,
+    /// A type of proxy or VPN that is known, but not in the above list of variants.
+    ///
+    /// This typically indicates that the geolocation database contains a proxy type that did not
+    /// exist when this crate was published.
+    Other(String),
+}
+
+use serde::Deserializer;
+
+fn deserialize_conn_speed<'de, D: Deserializer<'de>>(
+    deserializer: D,
+) -> Result<ConnSpeed, D::Error> {
+    deserialize_with_unknown(deserializer, ConnSpeed::Other)
+}
+
+fn deserialize_conn_type<'de, D: Deserializer<'de>>(deserializer: D) -> Result<ConnType, D::Error> {
+    deserialize_with_unknown(deserializer, ConnType::Other)
+}
+
+fn deserialize_continent<'de, D: Deserializer<'de>>(
+    deserializer: D,
+) -> Result<Continent, D::Error> {
+    deserialize_with_unknown(deserializer, Continent::Other)
+}
+
+fn deserialize_proxy_description<'de, D: Deserializer<'de>>(
+    deserializer: D,
+) -> Result<ProxyDescription, D::Error> {
+    deserialize_with_unknown(deserializer, ProxyDescription::Other)
+}
+
+fn deserialize_proxy_type<'de, D: Deserializer<'de>>(
+    deserializer: D,
+) -> Result<ProxyType, D::Error> {
+    deserialize_with_unknown(deserializer, ProxyType::Other)
+}
+
+fn deserialize_with_unknown<'de, D: Deserializer<'de>, T: Deserialize<'de>>(
+    deserializer: D,
+    catchall: fn(String) -> T,
+) -> Result<T, D::Error> {
+    let s: String = String::deserialize(deserializer)?;
+    use serde::de::value::StringDeserializer;
+    use serde::de::IntoDeserializer;
+    let deserializer: StringDeserializer<D::Error> = s.clone().into_deserializer();
+    Ok(T::deserialize(deserializer).unwrap_or_else(|_| catchall(s)))
+}
+
+#[test]
+fn deserialize_partial_geo_responses() {
+    let invalid_utc_offset = br#"
+    {
+        "as_name": "test as",
+        "as_number": 11111,
+        "area_code": 0,
+        "city": "?",
+        "conn_speed": "broadband",
+        "conn_type": "wired",
+        "continent": "OS",
+        "country_code": "EA",
+        "country_code3": "EAR",
+        "country_name": "the entire earth",
+        "latitude": 6.5,
+        "longitude": -28.8,
+        "metro_code": 0,
+        "postal_code": "?",
+        "proxy_description": "?",
+        "proxy_type": "hosting",
+        "region": "?",
+        "utc_offset": 9999
+    }
+    "#;
+    let deserialized = serde_json::from_slice::<'_, RawGeo>(invalid_utc_offset)
+        .ok()
+        .map(Geo::from_raw);
+    assert!(deserialized.is_some());
+    assert!(deserialized.unwrap().utc_offset.is_none());
+
+    let invalid_variant = br#"
+    {
+        "as_name": "test as",
+        "as_number": 11111,
+        "area_code": 0,
+        "city": "?",
+        "conn_speed": "super_broadband",
+        "conn_type": "wired",
+        "continent": "OS",
+        "country_code": "EA",
+        "country_code3": "EAR",
+        "country_name": "the entire earth",
+        "latitude": 6.5,
+        "longitude": -28.8,
+        "metro_code": 0,
+        "postal_code": "?",
+        "proxy_description": "?",
+        "proxy_type": "hosting",
+        "region": "?",
+        "utc_offset": 200
+    }
+    "#;
+    let deserialized = serde_json::from_slice::<'_, RawGeo>(invalid_variant)
+        .ok()
+        .map(Geo::from_raw);
+    assert!(deserialized.is_some());
+    assert_eq!(
+        deserialized.unwrap().conn_speed(),
+        ConnSpeed::Other("super_broadband".to_string())
+    );
 }
