@@ -217,13 +217,20 @@ impl Body {
     /// let json: serde_json::Value = serde_json::from_slice(&prefix).unwrap();
     /// ```
     pub fn get_prefix_mut(&mut self, length: usize) -> Prefix {
+        self.try_get_prefix_mut(length).expect("body read failed")
+    }
+
+    /// Try to get a prefix of the body up to the given number of bytes.
+    ///
+    /// Unlike [`get_prefix_mut()`][`Self::get_prefix_mut()`], this method does not panic if an I/O
+    /// error occurs.
+    pub fn try_get_prefix_mut(&mut self, length: usize) -> std::io::Result<Prefix> {
         let mut buf = vec![];
         let nread = self
             .take(length.try_into().unwrap())
-            .read_to_end(&mut buf)
-            .expect("body read failed");
+            .read_to_end(&mut buf)?;
         buf.truncate(nread);
-        Prefix::new(buf, self)
+        Ok(Prefix::new(buf, self))
     }
 
     /// Get a prefix of the body as a string containing up to the given number of bytes.
