@@ -29,8 +29,7 @@ impl StreamingBody {
 
     /// Append a body onto the end of this streaming body.
     ///
-    /// This operation is performed in amortized constant time, and so should always be preferred to
-    /// reading an entire body and then writing the same contents to another body.
+    #[doc = include_str!("../../../docs/snippets/body-append-constant-time.md")]
     ///
     /// # Examples
     ///
