@@ -1,6 +1,6 @@
 use super::{FastlyResponseMetadata, Response};
 use crate::abi::{self, FastlyStatus, MultiValueHostcallError};
-use crate::error::BufferSizeError;
+use crate::error::{BufferSizeError, HandleError, HandleKind};
 use crate::handle::{BodyHandle, StreamingBodyHandle};
 use crate::http::request::SendError;
 use crate::http::request::SendErrorCause;
@@ -8,6 +8,7 @@ use bytes::{BufMut, BytesMut};
 use http::header::{HeaderName, HeaderValue};
 use http::{StatusCode, Version};
 use std::convert::TryFrom;
+use std::mem::ManuallyDrop;
 
 // This import is just to get `RequestHandle` into scope for intradoc linking.
 #[allow(unused)]
@@ -55,13 +56,13 @@ impl ResponseHandle {
     };
 
     /// Returns `true` if the response handle is valid.
-    pub fn is_valid(&self) -> bool {
+    pub const fn is_valid(&self) -> bool {
         !self.is_invalid()
     }
 
     /// Returns `true` if the response handle is invalid.
-    pub fn is_invalid(&self) -> bool {
-        self.handle == Self::INVALID.handle
+    pub const fn is_invalid(&self) -> bool {
+        self.handle == fastly_shared::INVALID_RESPONSE_HANDLE
     }
 
     /// Get the underlying representation of the handle.
@@ -80,6 +81,14 @@ impl ResponseHandle {
         &mut self.handle
     }
 
+    /// Turn a handle into its representation without closing the underlying resource.
+    ///
+    /// This should only be used when calling the raw ABI directly, and care should be taken not to
+    /// reuse or alias handle values.
+    pub(crate) fn into_u32(self) -> u32 {
+        ManuallyDrop::new(self).as_u32()
+    }
+
     /// Acquire a new response handle.
     ///
     /// By default, the response will have a status code of `200 OK` and empty headers.
@@ -281,13 +290,7 @@ impl ResponseHandle {
         .expect("fastly_http_resp::header_values_set failed");
     }
 
-    /// Get the value of a header, or `None` if the header is not present.
-    ///
-    /// If there are multiple values for the header, only one is returned. See
-    /// [`get_header_values()`][`Self::get_header_values()`] if you need to get all of the values.
-    ///
-    /// If the value is longer than `max_len`, this will return a [`BufferSizeError`]; you can retry
-    /// with a larger buffer size if necessary.
+    #[doc = include_str!("../../../docs/snippets/handle-get-header-value.md")]
     pub fn get_header_value(
         &self,
         name: &HeaderName,
@@ -420,7 +423,7 @@ impl ResponseHandle {
     /// Immediately begin sending this response downstream to the client with the given body.
     pub fn send_to_client(self, body: BodyHandle) {
         unsafe {
-            abi::fastly_http_resp::send_downstream(self.as_u32(), body.as_u32(), false as u32)
+            abi::fastly_http_resp::send_downstream(self.into_u32(), body.into_u32(), false as u32)
         }
         .result()
         .expect("fastly_http_resp::send_downstream failed");
@@ -429,15 +432,39 @@ impl ResponseHandle {
     /// Immediately begin sending this response downstream to the client, and return a
     /// [`StreamingBodyHandle`] that can accept further data to send.
     pub fn stream_to_client(self, body: BodyHandle) -> StreamingBodyHandle {
-        let streaming_body_handle = StreamingBodyHandle::from_body_handle(&body);
         let status = unsafe {
-            abi::fastly_http_resp::send_downstream(self.as_u32(), body.as_u32(), true as u32)
+            abi::fastly_http_resp::send_downstream(self.into_u32(), body.as_u32(), true as u32)
         };
+        let streaming_body_handle = StreamingBodyHandle::from_body_handle(body);
         status
             .result()
             .map(|_| streaming_body_handle)
             .expect("fastly_http_resp::send_downstream failed")
     }
+
+    /// Close the ResponseHandle by removing it from the host Session. If the handle has already
+    /// been closed an error will be returned. A ResponseHandle is only consumed
+    /// when you send a response to a client or stream one to a client. You
+    /// should call close only if you don't intend to use that Response anymore.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use fastly::handle::ResponseHandle;
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// let response = ResponseHandle::new();
+    /// // The handle is not being used so we can close it out without any
+    /// // trouble
+    /// response.close()?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn close(self) -> Result<(), HandleError> {
+        match unsafe { abi::fastly_http_resp::close(self.as_u32()) } {
+            FastlyStatus::OK => Ok(()),
+            _ => Err(HandleError::ClosedHandle(HandleKind::Response)),
+        }
+    }
 }
 
 pub(crate) fn handles_to_response(
@@ -456,3 +483,13 @@ pub(crate) fn handles_to_response(
         )),
     }
 }
+
+impl Drop for ResponseHandle {
+    fn drop(&mut self) {
+        if self.is_valid() {
+            unsafe { abi::fastly_http_resp::close(self.as_u32()) }
+                .result()
+                .expect("fastly_http_resp::close failed");
+        }
+    }
+}
